#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <avr/delay.h>
#include <stdio.h>
#include <string.h>

#include "types.h"
#include "lcd.h"
#include "uart.h"
#include "enocean.h"
#include "keys.h"
#include "i2csw.h"
#include "relais.h"

void blink(void)
{
  sbi(DDRB,PB0);
  sbi(PORTB,PB0);
  _delay_ms(400); //400ms
  _delay_ms(400); //400ms
  cbi(DDRB,PB0);
  cbi(PORTB,PB0);
  _delay_ms(400); //400ms
  _delay_ms(400); //400ms
}

int main(void)
{
  struct EnoceanMsg msg;
  u08 retVal=0xff;

  lcd_Init();
  keys_Init();
  uart_Init();
  i2cInit();
  initRelais();

  lcd_gotoxy(0,1);
  lcd_puts("enocean radio switch\n");
  lcd_gotoxy(0,2);
 

  while(1)
  {	

    if( get_key_long( 1<<SELECT ))
	{
       lcd_clrscr();
	   lcd_home();
	   lcd_puts("Hauptmen\nSchalter verwalten?");
	}

    if( get_key_press( 1<<DOWN ))
	{
      lcd_gotoxy(0,2);
	  lcd_puts("Relais verwalten?  ");
	  setRelais(0,7);
	}

    if( get_key_press( 1<<UP ))
    {
	  lcd_gotoxy(0,2);
	  lcd_puts("Zurck             ");
	  releaseRelais(0,7);
	}

	if(enoceanPendingMsg())
	{ 
	  retVal=enoceanDecodeMsg(&msg);
	  if(retVal == OK)
	  {
        lcd_gotoxy(0,2);
		lcd_puts("ID ");
		lcd_puthex(msg.id[0]); 
		lcd_puthex(msg.id[1]);
		lcd_puthex(msg.id[2]);
		lcd_puthex(msg.id[3]);
		
		if(I_Button_Pressed(&msg))
		{
		  lcd_puts(" I-Button");
          sbi(DDRB,PB0);
          sbi(PORTB,PB0);
		  setRelais(0,1);
        }

		if(O_Button_Pressed(&msg))
		{
		  lcd_puts(" O-Button");
		  cbi(DDRB,PB0);
          cbi(PORTB,PB0);
		  changeRelais(0,0);
        }

		if(Button_Released(&msg))
        {
		  //released button
		  lcd_puts(" Released");
		  releaseRelais(0,1);
        }
      }
      else
	  {
	    lcd_gotoxy(0,2);
		lcd_puts("Dekodieren NOK: ");
		lcd_puthex(retVal);
      } 
	}  		 
  }
  return(0);

}
