#include "enocean.h"
#include "uart.h"

u08 enoceanDecodeMsg(struct EnoceanMsg* msg)
{
  u08 checksum,calc_checksum,byte;
  u08 synch=0;

  //synchronise to msg begin
  while(uart_Waiting_Bytes() > 0)
  {
  	byte=uart_Dequeue_Byte();

	if(byte == 0xa5)
    {
	  synch=1;
	  continue;
    } 
    if((byte == 0x5a) && (synch == 1))
	{
	  synch=2;
	  break;
    } else {
	  synch=0;
	  continue; 
    }
  }
  if(synch!=2)
    return(SYNCH_FAILED);

  byte=uart_Dequeue_Byte();
  msg->h_seq= (byte & 0x70) >> 5;
  msg->length= byte & 0x1f;
  
  msg->org=uart_Dequeue_Byte();;
  msg->data[0]=uart_Dequeue_Byte();
  msg->data[1]=uart_Dequeue_Byte();
  msg->data[2]=uart_Dequeue_Byte();
  msg->data[3]=uart_Dequeue_Byte();
  msg->id[0]=uart_Dequeue_Byte();
  msg->id[1]=uart_Dequeue_Byte();
  msg->id[2]=uart_Dequeue_Byte();
  msg->id[3]=uart_Dequeue_Byte();
  msg->status=uart_Dequeue_Byte();
  checksum=uart_Dequeue_Byte();
  calc_checksum=byte+msg->org+msg->data[0]+msg->data[1]+msg->data[2]+
                msg->data[3]+msg->id[0]+msg->id[1]+msg->id[2]+msg->id[3]+
                msg->status;
  if(checksum==calc_checksum)
  	return(OK);
  else
    return(CHECK_FAILED);
}

bool I_Button_Pressed(struct EnoceanMsg* msg)
{
  if(msg->org == 0x05) //PTM switch msg?
    if((msg->status & 0x10)>>4) //NU=1? (Button pressed?)
	  return(((msg->data[0] & 0x20)>>5) == 0x00); //UD=0? (I-Button)
    
  return(false);
}

bool O_Button_Pressed(struct EnoceanMsg* msg)
{
  if(msg->org == 0x05) //PTM switch msg?
    if((msg->status & 0x10)>>4) //NU=1? (Button pressed?)
	  return(((msg->data[0] & 0x20)>>5) == 0x01); //UD=1? (O-Button)
    
  return(false);
}

bool Button_Released(struct EnoceanMsg* msg)
{
  if(msg->org == 0x05) //PTM switch msg?
    if(((msg->status & 0x10)>>4) == 0x00) //NU=0? (Button released?)
	  return(((msg->data[0] & 0xf0)>>4) == 0x00); //0 Buttons & PR=0

  return(false);
}

bool enoceanPendingMsg(void)
{
  return(uart_Waiting_Bytes()>=14);
}
