#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/delay.h>

#include "types.h"
#include "lcd.h"

#define  LCD_CGRAM 6

u08 hstring[16] = "0123456789ABCDEF"; //used by lcd_puthex

// custom LCD characters
static unsigned char __attribute__ ((progmem)) LcdCustomChar[] =
{
  0x00, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, // 0. 0/5 full progress block
  0x00, 0x1F, 0x10, 0x10, 0x10, 0x10, 0x1F, 0x00, // 1. 1/5 full progress block
  0x00, 0x1F, 0x18, 0x18, 0x18, 0x18, 0x1F, 0x00, // 2. 2/5 full progress block
  0x00, 0x1F, 0x1C, 0x1C, 0x1C, 0x1C, 0x1F, 0x00, // 3. 3/5 full progress block
  0x00, 0x1F, 0x1E, 0x1E, 0x1E, 0x1E, 0x1F, 0x00, // 4. 4/5 full progress block
  0x00, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x00, // 5. 5/5 full progress block
  0x03, 0x07, 0x0F, 0x1F, 0x0F, 0x07, 0x03, 0x00, // 6. rewind arrow
  0x00, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x00, // 7. stop block
  0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x1B, 0x00, // 8. pause bars
  0x18, 0x1C, 0x1E, 0x1F, 0x1E, 0x1C, 0x18, 0x00, // 9. fast-forward arrow
  0x00, 0x04, 0x04, 0x0E, 0x0E, 0x1F, 0x1F, 0x00, // 10. scroll up arrow
  0x00, 0x1F, 0x1F, 0x0E, 0x0E, 0x04, 0x04, 0x00, // 11. scroll down arrow
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 12. blank character
  0x00, 0x0E, 0x19, 0x15, 0x13, 0x0E, 0x00, 0x00, // 13. animated play icon frame 0
  0x00, 0x0E, 0x15, 0x15, 0x15, 0x0E, 0x00, 0x00, // 14. animated play icon frame 1
  0x00, 0x0E, 0x13, 0x15, 0x19, 0x0E, 0x00, 0x00, // 15. animated play icon frame 2
  0x00, 0x0E, 0x11, 0x1F, 0x11, 0x0E, 0x00, 0x00, // 16. animated play icon frame 3
};

static u08 lcd_x, lcd_y;
/* local functions */
void lcd_waitbusy(void)
{
  register u08 i = 0;
  register u08 data;

  sbi(LCD_FUNC_PORT,LCD_RW_PIN);      // R/W -> 1
  cbi(LCD_FUNC_PORT,LCD_RS_PIN);      // RS  -> 0
  outp(0x00, LCD_DATA_PORT_DDR);      // set port as input
  outp(0xff, LCD_DATA_PORT);          // enable pullup
  do
  {
     lcd_e_high();                    // set LCD enable high
     data = inp(LCD_DATA_PORT);       // read byte
     lcd_e_low();                     // set LCD enable low
     i++;
  } while((data & (1 << LCD_BUSY)) && (i != 255));    

  cbi(LCD_FUNC_PORT,LCD_RW_PIN);      // R/W -> 0
}

static void lcd_newline(void)         // goto start of next line
{
    lcd_x = 0;
    if (lcd_y < LCD_LINES)
        lcd_y++;
}

void lcd_goto(void)                   // goto position (lcd_x,lcd_y)
{
 static u08 lcd_line[] = {0x00, 0x40, 0x14, 0x54};
 lcd_write((0x80 + lcd_line[lcd_y] + lcd_x),0);
}

void lcd_gotoxy(u08 x,u08 y)          // goto given position (x&y counts from 1)
{
  lcd_x=x;
  lcd_y=y-1;                          // y counts from 0
  lcd_goto();
}

/* public functions */
void lcd_Init(void)
{
  static u16 wait[] = {16000,5000,70,70,40,1600,40};
  static u08 init[] = {0x38,0x38,0x38,0x38,0x0c,0x01,0x06};
  cbi(MCUCR, SRE);                    //disable external RAM
  outp(0xff,LCD_DATA_PORT_DDR);
  outp(0xff,LCD_FUNC_PORT_DDR);
  outp(0x00,LCD_FUNC_PORT);
  _delay_ms(50);                      //wait more than 40ms

  register u08 i=0;
  for(i=0;i<=6;i++)                   //init display
  {
    outp(init[i],LCD_DATA_PORT);
    lcd_e_cycle();
    _delay_us(wait[i]);
  }
  lcd_loadCustomChar((u08*)LcdCustomChar,0,0);
  lcd_loadCustomChar((u08*)LcdCustomChar,1,1);
  lcd_loadCustomChar((u08*)LcdCustomChar,2,2);
  lcd_loadCustomChar((u08*)LcdCustomChar,3,3);
  lcd_loadCustomChar((u08*)LcdCustomChar,4,4);
  lcd_loadCustomChar((u08*)LcdCustomChar,5,5);
  lcd_loadCustomChar((u08*)LcdCustomChar,6,6);
  lcd_loadCustomChar((u08*)LcdCustomChar,7,7);
  lcd_loadCustomChar((u08*)LcdCustomChar,8,8);
  lcd_loadCustomChar((u08*)LcdCustomChar,9,9);
  lcd_loadCustomChar((u08*)LcdCustomChar,10,10);
  lcd_loadCustomChar((u08*)LcdCustomChar,11,11);
  lcd_loadCustomChar((u08*)LcdCustomChar,12,12);
  lcd_loadCustomChar((u08*)LcdCustomChar,13,13);
  lcd_loadCustomChar((u08*)LcdCustomChar,14,14);
  lcd_loadCustomChar((u08*)LcdCustomChar,15,15);
}

void lcd_write(u08 data,u08 rs)
{
  lcd_waitbusy();

  cbi(LCD_FUNC_PORT,LCD_RW_PIN);      // R/W -> 0
  if(rs)                              // RS? 1 -> command, 0 -> data
    sbi(LCD_FUNC_PORT,LCD_RS_PIN);

  outp(0xff,LCD_DATA_PORT_DDR);       // set port as output
  outp(data,LCD_DATA_PORT);
  lcd_e_cycle();
}

void lcd_putchar(u08 data)            // print character to
{                                     // current cursor position
  switch (data)                       // convert german umlauts
  {
    case 0xf6:
    case 0xd6: data = 0xef;           // convert  & 
               break;
    case 0xe4:
    case 0xc4: data = 0xe1;           // convert  & 
               break;
    case 0xfc:
    case 0xdc: data = 0xf5;           // convert  & 
               break;
    case 0xdf: data = 0xe2;           // convert 
               break;
  }  	
  if (data == '\n')
  {
     lcd_newline();
     lcd_goto();
  }
  else
  {
     if (lcd_x < LCD_LINE_LENGTH)
     {
        lcd_x++;
        lcd_write(data,1);
     }
  }
}

void lcd_puts(u08* str)                // print string on lcd 
{                                      //  (no auto linefeed)
        while (*str)
                lcd_putchar(*str++);
}

void lcd_puthex(u08 data)
{
	//print high nibble as hex
	lcd_putchar(hstring[data>>4]);
	//print low nibble as hex
	lcd_putchar(hstring[data & 0x0F]);
}

void lcd_clrscr(void)                  // clear lcd
{
        lcd_x = lcd_y = 0;
        lcd_write(0x01,0);             // clear lcd
        _delay_ms(3);                  // 3 ms delay
}

void lcd_home(void)                    // set cursor
{                                      //  to home position
        lcd_x = lcd_y = 0;
        lcd_write(0x02,0);
        _delay_ms(2);                  // 2 ms delay
}

void lcd_loadCustomChar(u08* lcdCustomCharArray, u08 romCharNum, u08 lcdCharNum)
{
	register u08 i;

	// multiply the character index by 8
	lcdCharNum = (lcdCharNum<<3);	   // each character occupies 8 bytes
	romCharNum = (romCharNum<<3);	   // each character occupies 8 bytes

	// copy the 8 bytes into CG (character generator) RAM
	for(i=0; i<8; i++)
	{
		// set CG RAM address
		lcd_write(((1<<LCD_CGRAM) | (lcdCharNum+i)),0);
		// write character data
		lcd_write( pgm_read_byte(lcdCustomCharArray+romCharNum+i),1 );
	}
}
