#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/delay.h>

#include "uart.h"

u08 rx_buffer[RX_BUFFER_LENGTH];
u08 rx_lastRecIndex=0;
u08 rx_lastProIndex=0;

SIGNAL(SIG_UART_RECV)
{
   uart_Enqueue_Byte(UDR);
}

inline void uart_Enqueue_Byte(u08 byte)
{
  rx_lastRecIndex = (rx_lastRecIndex+1) % RX_BUFFER_LENGTH;
  rx_buffer[rx_lastRecIndex] = byte;
}

u08 uart_Dequeue_Byte(void)
{
  rx_lastProIndex = (rx_lastProIndex+1) % RX_BUFFER_LENGTH;
  return rx_buffer[rx_lastProIndex];
}

u08 uart_Waiting_Bytes(void)
{
  if(rx_lastRecIndex>=rx_lastProIndex)
    return(rx_lastRecIndex-rx_lastProIndex);
  else
    return(RX_BUFFER_LENGTH+rx_lastRecIndex-rx_lastProIndex);
}

void uart_Init(void)
{
  //Initialization UART
  
  sbi( PORTE,PE0);   // enable pullup
  UBRR = UART_BAUD_CALC(UART_BAUD_RATE); //set BAUD Rate
  UCR |= (1<<RXEN);  //enable RxD
  UCR |= (1<<RXCIE); //enable RxD complete interrupt

  sei();
}
